@@ -20,6 +20,9 @@ class AtomicInt:
         self._value = initial_value
         self._lock: Lock = Lock()
 
+    def __str__(self):
+        return f"AtomicInt[value={self._value}]"
+
     def get(self):
         with self._lock:
             return self._value
@@ -49,3 +52,10 @@ class AtomicInt:
         with self._lock:
             self._value -= 1
             return self._value
+
+    def compare_and_set(self, expected_value: int, new_value: int) -> bool:
+        with self._lock:
+            if self._value == expected_value:
+                self._value = new_value
+                return True
+            return False
